@@ -5,6 +5,7 @@
 #include "cbc.h"
 #include "ctr.h"
 #include "knuth-lfib.h"
+#include "nettle-internal.h"
 
 #include <ctype.h>
 #include <stdio.h>
@@ -225,8 +226,16 @@ test_cipher_cbc(const struct nettle_cipher *cipher,
 	      length, data, cleartext);
 
   if (!MEMEQ(length, data, ciphertext))
-    FAIL();
-
+    {
+      fprintf(stderr, "CBC encrypt failed:\nInput:");
+      print_hex(length, cleartext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, ciphertext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
   cipher->set_decrypt_key(ctx, key_length, key);
   memcpy(iv, iiv, cipher->block_size);
 
@@ -235,8 +244,16 @@ test_cipher_cbc(const struct nettle_cipher *cipher,
 	      length, data, data);
 
   if (!MEMEQ(length, data, cleartext))
-    FAIL();
-
+    {
+      fprintf(stderr, "CBC decrypt failed:\nInput:");
+      print_hex(length, ciphertext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, cleartext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
   free(ctx);
   free(data);
   free(iv);
@@ -263,7 +280,16 @@ test_cipher_ctr(const struct nettle_cipher *cipher,
 	    length, data, cleartext);
 
   if (!MEMEQ(length, data, ciphertext))
-    FAIL();
+    {
+      fprintf(stderr, "CTR encrypt failed:\nInput:");
+      print_hex(length, cleartext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, ciphertext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
 
   memcpy(ctr, ictr, cipher->block_size);
 
@@ -272,7 +298,16 @@ test_cipher_ctr(const struct nettle_cipher *cipher,
 	    length, data, data);
 
   if (!MEMEQ(length, data, cleartext))
-    FAIL();
+    {
+      fprintf(stderr, "CTR decrypt failed:\nInput:");
+      print_hex(length, ciphertext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, cleartext);
+      fprintf(stderr, "\n");
+      FAIL();
+    }
 
   free(ctx);
   free(data);
@@ -305,12 +340,22 @@ test_cipher_stream(const struct nettle_cipher *cipher,
 	  if (data[i + block] != 0x17)
 	    FAIL();
 	}
+
       cipher->encrypt(ctx, length - i, data + i, cleartext + i);
       if (data[length] != 0x17)
 	FAIL();
       
       if (!MEMEQ(length, data, ciphertext))
-	FAIL();
+	{
+	  fprintf(stderr, "Encrypt failed, block size %d\nInput:", block);
+	  print_hex(length, cleartext);
+	  fprintf(stderr, "\nOutput: ");
+	  print_hex(length, data);
+	  fprintf(stderr, "\nExpected:");
+	  print_hex(length, ciphertext);
+	  fprintf(stderr, "\n");
+	  FAIL();	    
+	}
     }
   
   cipher->set_decrypt_key(ctx, key_length, key);
@@ -320,10 +365,79 @@ test_cipher_stream(const struct nettle_cipher *cipher,
     FAIL();
 
   if (!MEMEQ(length, data, cleartext))
+    {
+      fprintf(stderr, "Decrypt failed\nInput:");
+      print_hex(length, ciphertext);
+      fprintf(stderr, "\nOutput: ");
+      print_hex(length, data);
+      fprintf(stderr, "\nExpected:");
+      print_hex(length, cleartext);
+      fprintf(stderr, "\n");
+      FAIL();	    
+    }
+
+  free(ctx);
+  free(data);
+}
+
+void
+test_aead(const struct nettle_aead *aead,
+	  unsigned key_length,
+	  const uint8_t *key,
+	  unsigned auth_length,
+	  const uint8_t *authtext,
+	  unsigned length,
+	  const uint8_t *cleartext,
+	  const uint8_t *ciphertext,
+	  unsigned iv_length,
+	  const uint8_t *iv,
+	  const uint8_t *digest)
+{
+  void *ctx = xalloc(aead->context_size);
+  uint8_t *data = xalloc(length);
+  uint8_t *buffer = xalloc(aead->block_size);
+
+  /* encryption */
+  memset(buffer, 0, aead->block_size);
+  aead->set_key(ctx, key_length, key);
+
+  aead->set_iv(ctx, iv_length, iv);
+
+  if (auth_length)
+    aead->update(ctx, auth_length, authtext);
+    
+  if (length)
+    aead->encrypt(ctx, length, data, cleartext);
+
+  aead->digest(ctx, aead->block_size, buffer);
+
+  if (!MEMEQ(length, data, ciphertext))
+    FAIL();
+
+  if (!MEMEQ(aead->block_size, buffer, digest))
+    FAIL();
+
+  /* decryption */
+  memset(buffer, 0, aead->block_size);
+  aead->set_iv(ctx, iv_length, iv);
+
+  if (auth_length)
+    aead->update(ctx, auth_length, authtext);
+    
+  if (length)
+    aead->decrypt(ctx, length, data, data);
+
+  aead->digest(ctx, aead->block_size, buffer);
+
+  if (!MEMEQ(length, data, cleartext))
+    FAIL();
+
+  if (!MEMEQ(aead->block_size, buffer, digest))
     FAIL();
 
   free(ctx);
   free(data);
+  free(buffer);
 }
 
 void
